using System;
using System.Collections.Generic;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Collections.ObjectModel;

using DomainGenie;

namespace ExtendedWebBrowser2
{
    public delegate void WebActionHandler(object sender,WebActionEventArgs e);

    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1301:AvoidDuplicateAccelerators")]
    partial class MultiTabBrowser : UserControl
    {
        [Description("Event fires when the Thumb moves")]
        [Category("Behavior"), Browsable(false)]
        public event WebActionHandler WebAction;


        FavoritesSelector fs = null;
        public MultiTabBrowser()
        {
            InitializeComponent();
            LoadFavorites();
            _windowManager = new WindowManager(this.tabControl);
            _windowManager.CommandStateChanged += new EventHandler<CommandStateEventArgs>(_windowManager_CommandStateChanged);
            _windowManager.StatusTextChanged += new EventHandler<TextChangedEventArgs>(_windowManager_StatusTextChanged);

        }

        private void LoadFavorites()
        {
            fs = new FavoritesSelector();
            fs.TreeView.NodeMouseClick += new TreeNodeMouseClickEventHandler(TreeView_NodeMouseClick);
            fs.TreeView.AfterExpand += new TreeViewEventHandler(TreeView_AfterExpand);
            fs.Dock = DockStyle.Fill;
            fs.InitNodes();
            //fs.MenuStrip.ForeColor = _crtFColM;

            splitContainer1.Panel1.Controls.Add(fs);
        }

        private int _expandTick;
        void TreeView_AfterExpand(object sender, TreeViewEventArgs e)
        {
            _expandTick = Environment.TickCount;
        }

        void TreeView_NodeMouseClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            if (_expandTick + 1000 > Environment.TickCount) return;
            if (e.Node != null && !e.Node.Bounds.Contains(e.Location))
            {
                return;
            }

            string url = fs.GetUrl(e.Node);
            if((url.Length > 0) && (tabControl.Visible))
            {
                _windowManager.ActiveBrowser.Navigate(url);
                fs.Browser = _windowManager.ActiveBrowser;
            }

        }

        // Update the status text
        void _windowManager_StatusTextChanged(object sender, TextChangedEventArgs e)
        {
            //this.toolStripStatusLabel.Text = e.Text;
        }

        // Enable / disable buttons
        void _windowManager_CommandStateChanged(object sender, CommandStateEventArgs e)
        {
            this.forwardToolStripButton.Enabled = ((e.BrowserCommands & BrowserCommands.Forward) == BrowserCommands.Forward);
            this.backToolStripButton.Enabled = ((e.BrowserCommands & BrowserCommands.Back) == BrowserCommands.Back);
            this.printPreviewToolStripButton.Enabled = ((e.BrowserCommands & BrowserCommands.PrintPreview) == BrowserCommands.PrintPreview);
            //this.printPreviewToolStripMenuItem.Enabled = ((e.BrowserCommands & BrowserCommands.PrintPreview) == BrowserCommands.PrintPreview);
            this.printToolStripButton.Enabled = ((e.BrowserCommands & BrowserCommands.Print) == BrowserCommands.Print);
            //this.printToolStripMenuItem.Enabled = ((e.BrowserCommands & BrowserCommands.Print) == BrowserCommands.Print);
            this.homeToolStripButton.Enabled = ((e.BrowserCommands & BrowserCommands.Home) == BrowserCommands.Home);
            this.searchToolStripButton.Enabled = ((e.BrowserCommands & BrowserCommands.Search) == BrowserCommands.Search);
            this.refreshToolStripButton.Enabled = ((e.BrowserCommands & BrowserCommands.Reload) == BrowserCommands.Reload);
            this.stopToolStripButton.Enabled = ((e.BrowserCommands & BrowserCommands.Stop) == BrowserCommands.Stop);
        }

        #region Tools menu
        // Executed when the user clicks on Tools -> Options
        private void optionsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //using (OptionsForm of = new OptionsForm())
            //{
            //    of.ShowDialog(this);
            //}
        }
        // Tools -> Show script errors
        private void scriptErrorToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ScriptErrorManager.Instance.ShowWindow();
        }

        #endregion

        #region File Menu

        // File -> Print
        private void printToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Print();
        }

        // File -> Print Preview
        private void printPreviewToolStripMenuItem_Click(object sender, EventArgs e)
        {
            PrintPreview();
        }

        // File -> Exit
        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //this.Close();
        }

        // File -> Open URL
        private void openUrlToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //using (OpenUrlForm ouf = new OpenUrlForm())
            //{
            //  if (ouf.ShowDialog() == DialogResult.OK)
            //  {
            //    ExtendedWebBrowser brw = _windowManager.New(false);
            //    brw.Navigate(ouf.Url);
            //  }
            //}
        }

        // File -> Open File
        private void openFileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            using (OpenFileDialog ofd = new OpenFileDialog())
            {
                ofd.Filter = "Html files (*.htm & *.html)|*.htm;*.html|All files (*.*)|*.*";
                if (ofd.ShowDialog() == DialogResult.OK)
                {
                    Uri url = new Uri(ofd.FileName);
                    WindowManager.Open(url);
                    fs.Browser = _windowManager.ActiveBrowser;
                }
            }
        }
        #endregion

        #region Help Menu

        // Executed when the user clicks on Help -> About
        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            About();
        }

        /// <summary>
        /// Shows the AboutForm
        /// </summary>
        private void About()
        {
            //using (AboutForm af = new AboutForm())
            //{
            //  af.ShowDialog(this);
            //}
        }

        #endregion


        /// <summary>
        /// The WindowManager class
        /// </summary>
        private WindowManager _windowManager;

        // This is handy when all the tabs are closed.
        private void tabControl_VisibleChanged(object sender, EventArgs e)
        {
            //if (tabControl.Visible)
            //{
            //    this.panel1.BackColor = SystemColors.Control;
            //}
            //else
            //    this.panel1.BackColor = SystemColors.AppWorkspace;
        }

        // Starting the app here...
        private void MultiTabBrowser_Load(object sender, EventArgs e)
        {
            Uri url = new Uri("http://www.domain-genie.com");
            _windowManager.Open(url);
            fs.Browser = _windowManager.ActiveBrowser;
        }


        public void LoadBrowser(string url)
        {
            _windowManager.ActiveBrowser.Navigate(url);
            fs.Browser = _windowManager.ActiveBrowser;
        }


        #region Printing & Print Preview
        private void Print()
        {
            ExtendedWebBrowser brw = _windowManager.ActiveBrowser;
            if (brw != null)
                brw.ShowPrintDialog();
        }

        private void PrintPreview()
        {
            ExtendedWebBrowser brw = _windowManager.ActiveBrowser;
            if (brw != null)
                brw.ShowPrintPreviewDialog();
        }
        #endregion

        #region Toolstrip buttons
        private void closeWindowToolStripButton_Click(object sender, EventArgs e)
        {
            this._windowManager.New();
        }

        private void closeToolStripButton_Click(object sender, EventArgs e)
        {
            this._windowManager.Close();
        }

        private void printToolStripButton_Click(object sender, EventArgs e)
        {
            Print();
        }

        private void printPreviewToolStripButton_Click(object sender, EventArgs e)
        {
            PrintPreview();
        }

        private void backToolStripButton_Click(object sender, EventArgs e)
        {
            if (_windowManager.ActiveBrowser != null && _windowManager.ActiveBrowser.CanGoBack)
                _windowManager.ActiveBrowser.GoBack();
        }

        private void forwardToolStripButton_Click(object sender, EventArgs e)
        {
            if (_windowManager.ActiveBrowser != null && _windowManager.ActiveBrowser.CanGoForward)
                _windowManager.ActiveBrowser.GoForward();
        }

        private void stopToolStripButton_Click(object sender, EventArgs e)
        {
            if (_windowManager.ActiveBrowser != null)
            {
                _windowManager.ActiveBrowser.Stop();
            }
            stopToolStripButton.Enabled = false;
        }

        private void refreshToolStripButton_Click(object sender, EventArgs e)
        {
            if (_windowManager.ActiveBrowser != null)
            {
                _windowManager.ActiveBrowser.Refresh(WebBrowserRefreshOption.Normal);
            }
        }

        private void homeToolStripButton_Click(object sender, EventArgs e)
        {
            if (_windowManager.ActiveBrowser != null)
                _windowManager.ActiveBrowser.GoHome();
        }

        private void searchToolStripButton_Click(object sender, EventArgs e)
        {
            if (_windowManager.ActiveBrowser != null)
                _windowManager.ActiveBrowser.GoSearch();
        }

        #endregion



        public WindowManager WindowManager
        {
            get { return _windowManager; }
        }

        private void tsBtnGo_Click(object sender, EventArgs e)
        {
            _windowManager.ActiveBrowser.Navigate(comboURL.Text.TrimEnd());

            //splitContainer1.Panel1Collapsed = true;
            //splitContainer1.SplitterDistance = 245;

        }

        // Used for catching the Enter key in the textbox
        private void comboURL_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                e.Handled = true;
                _windowManager.ActiveBrowser.Navigate(this.comboURL.Text.TrimEnd());
            }
        }

        private void favoritesToolStripButton_Click(object sender, EventArgs e)
        {
            splitContainer1.Panel1Collapsed = !splitContainer1.Panel1Collapsed;
        }

        private void minmaxToolStripButton_Click(object sender, EventArgs e)
        {
            if (WebAction != null)
                WebAction(this, new WebActionEventArgs("minmax", ""));
        }

        public void ChangeIcon(int iDirection)
        {
            if (iDirection < 1)
            {
                minmaxToolStripButton.Image = global::DomainGenie.Properties.Resources.up;
                minmaxToolStripButton.ToolTipText = "Maximize";
            }
            else
            {
                minmaxToolStripButton.Image = global::DomainGenie.Properties.Resources.down;
                minmaxToolStripButton.ToolTipText = "Minimize";
            }
        }

    }//END OF CLASS


    class NotifyCollection<T> : Collection<T>
    {
        public event EventHandler CollectionChanged;
        protected virtual void OnCollectionChanged(EventArgs e)
        {
            if (CollectionChanged != null)
                CollectionChanged(this, e);
        }

        protected override void ClearItems()
        {
            base.ClearItems();
            OnCollectionChanged(EventArgs.Empty);
        }

        protected override void InsertItem(int index, T item)
        {
            base.InsertItem(index, item);
            OnCollectionChanged(EventArgs.Empty);
        }

        protected override void RemoveItem(int index)
        {
            base.RemoveItem(index);
            OnCollectionChanged(EventArgs.Empty);
        }

        protected override void SetItem(int index, T item)
        {
            base.SetItem(index, item);
            OnCollectionChanged(EventArgs.Empty);
        }

    }//END OF CLASS





    [Flags]
    enum BrowserCommands
    {
        /// <summary>
        /// Used when no commans are available
        /// </summary>
        None = 0,
        Home = 1,
        Search = 2,
        Back = 4,
        Forward = 8,
        Stop = 16,
        Reload = 32,
        Print = 64,
        PrintPreview = 128
    }

    /// <summary>
    /// Represents event information for the main form, when the command state of the active browser changes
    /// </summary>
    class CommandStateEventArgs : EventArgs
    {
        /// <summary>
        /// Creates a new instance of the <see cref="CommandStateEventArgs"/> class
        /// </summary>
        /// <param name="commands">A list of commands that are available</param>
        public CommandStateEventArgs(BrowserCommands commands)
        {
            _commands = commands;
        }
        private BrowserCommands _commands;

        /// <summary>
        /// Gets a list of commands that are available
        /// </summary>
        public BrowserCommands BrowserCommands
        {
            get { return _commands; }
        }
    }//END OF CLASS





    enum PopupBlockerFilterLevel
    {
        /// <summary>
        /// No pop-ups are blocked
        /// </summary>
        None = 0,
        /// <summary>
        /// Pop-ups of secure sites are allowed
        /// </summary>
        Low,
        /// <summary>
        /// Most pop-ups are blocked, unless the Ctrl key is pressed
        /// </summary>
        Medium,
        /// <summary>
        /// All pop-ups are blocked, unless the Ctrl key is pressed
        /// </summary>
        High
    }



    class ScriptErrorManager
    {
        private ScriptErrorManager()
        {
            _scriptErrors = new NotifyCollection<ScriptError>();
        }

        private NotifyCollection<ScriptError> _scriptErrors;

        private static object lockObject = new object();
        private static ScriptErrorManager _instance;

        public static ScriptErrorManager Instance
        {
            get
            {
                if (_instance == null)
                {
                    lock (lockObject)
                    {
                        if (_instance == null)
                            _instance = new ScriptErrorManager();
                    }
                }
                return _instance;
            }
        }

        public NotifyCollection<ScriptError> ScriptErrors
        {
            get
            {
                return _scriptErrors;
            }
        }

        //private ScriptErrorWindow _errorWindow;

        public void RegisterScriptError(Uri url, string description, int lineNumber)
        {
            //this._scriptErrors.Add(new ScriptError(url, description, lineNumber));
            //if (SettingsHelper.Current.ShowScriptErrors)
            //{
            //    ShowWindow();
            //}
        }

        public void ShowWindow()
        {
            //if (_errorWindow == null || _errorWindow.IsDisposed)
            //{
            //    _errorWindow = new ScriptErrorWindow();
            //}
            //_errorWindow.Show();
        }
    }



    class ScriptError
    {
        public ScriptError(Uri url, string description, int lineNumber)
        {
            _url = url;
            _description = description;
            _lineNumber = lineNumber;
        }

        private int _lineNumber;
        public int LineNumber
        {
            get { return _lineNumber; }
        }

        private string _description;
        public string Description
        {
            get { return _description; }
        }

        private Uri _url;
        public Uri Url
        {
            get { return _url; }
        }

    }//END OF CLASS



    class TextChangedEventArgs : EventArgs
    {
        public TextChangedEventArgs(string text)
        {
            _text = text;
        }

        string _text;
        public string Text
        {
            get { return _text; }
        }
    }//END OF CLASS

    public class WebActionEventArgs : EventArgs
    {
        public WebActionEventArgs(string sAction, string sValue)
        {
            this.sAction = sAction;
            this.sValue = sValue;
        }

        public string sAction;
        public string sValue;

    }    //end of class FireEventArgs



    /// <summary>
    /// Flags used by INewWindowManager::EvaluateNewWindow. 
    /// These values are taken into account in the decision of whether to display a pop-up window.
    /// </summary>
    [Flags]
    enum UrlContext
    {
        /// <summary>
        /// No information Present
        /// </summary>
        None = 0x0,
        /// <summary>
        /// The page is unloading. This flag is set in response to the onbeforeunload and onunload events. 
        /// Some pages load pop-up windows when you leave them rather than when you enter. This flag is used to identify those situations.
        /// </summary>
        Unloading = 0x1,
        /// <summary>
        /// The call to INewWindowManager::EvaluateNewWindow is the result of a user-initiated action 
        /// (a mouse click or key press). Use this flag in conjunction with the NWMF_FIRST_USERINITED flag 
        /// to determine whether the call is a direct or indirect result of the user-initiated action.
        /// </summary>
        UserInited = 0x2,
        /// <summary>
        /// When NWMF_USERINITED is present, this flag indicates that the call to 
        /// INewWindowManager::EvaluateNewWindow is the first query that results from this user-initiated action. 
        /// Always use this flag in conjunction with NWMF_USERINITED.
        /// </summary>
        UserFirstInited = 0x4,
        /// <summary>
        /// The override key (ALT) was pressed. The override key is used to bypass the pop-up managerallowing 
        /// all pop-up windows to displayand must be held down at the time that INewWindowManager::EvaluateNewWindow is called. 
        /// </summary>
        OverrideKey = 0x8,
        /// <summary>
        /// The new window attempting to load is the result of a call to the showHelp method. Help is sometimes displayed in a separate window, 
        /// and this flag is valuable in those cases.
        /// </summary>
        ShowHelp = 0x10,
        /// <summary>
        /// The new window is a dialog box that displays HTML content.
        /// </summary>
        HtmlDialog = 0x20,
        /// <summary>
        /// Indicates that the EvaluateNewWindow method is being called through a marshalled Component Object Model (COM) proxy 
        /// from another thread. In this situation, the method should make a decision and return immediately without performing 
        /// blocking operations such as showing modal user interface (UI). Lengthy operations will cause the calling thread to 
        /// appear unresponsive.
        /// </summary>
        FromProxy = 0x40
    }//END OF CLASS








}





//public delegate void NavigateHandler(object sender, NavigateEventArgs e);
//[Description("Event fires when the Thumb moves")]
//[Category("Behavior"), Browsable(false)]
//public event NavigateHandler Navigate;
//public class NavigateEventArgs : EventArgs
//{
//    public NavigateEventArgs(string url, bool bString)
//    {
//        this.url = url;
//        this.bString = bString;
//    }

//    public string url;
//    public bool bString;

//}    //end of class FireEventArgs

//if (Navigate != null)
//    Navigate(this, new NavigateEventArgs("http://www.software-rus.com/", false));




