using System;
using System.Net;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;

namespace NameGenerator
{
    class NameGenerator
    {
        private Random random;
        private char[] consonants;
        private char[] vowels;

        public NameGenerator()
        {
            random = new Random();
            vowels = new char[] { 'a', 'e', 'i', 'o', 'u', 'y' };
            consonants = new char[] { 
                'b', 'c', 'd', 'f', 'g', 'h', 'j', 'k', 'l', 'm',
                'n', 'p', 'q', 'r', 's', 't', 'v', 'w', 'x', 'z'
                };

        }

        /// <summary>
        /// Generates a nickname of specified length 
        /// </summary>
        /// <param name="length">Length of the nickname</param>
        /// <returns>Generated nickname</returns>
        public string Generate(int length, string startsWith)
        {
            StringBuilder sb = new StringBuilder();
            //initialize our vowel/consonant flag
            bool flag = (random.Next(2) == 0);

            for (int i = 0; i < length; i++)
            {
                sb.Append(GetChar(flag));
                flag = !flag; //invert the vowel/consonant flag
            }
            return startsWith+sb.ToString();
        }

        /// <summary>
        /// Gets number of results for nickname
        /// </summary>
        /// <param name="text"></param>
        /// <returns></returns>
        public static int GetWebResultsForNickname(string text)
        {
            string address = "http://www.google.com/xhtml/search?mrestrict=xhtml&hl=en&q=" + text;
            WebClient client = new WebClient();
            client.Encoding = Encoding.UTF8;
            string input = client.DownloadString(address);
            //regex 
            Match match = new Regex("Results [0-9] - [0-9]+ of .*?([0-9|,]+)\\.").Match(input);
            if (match.Success)
            {
                return int.Parse(match.Groups[1].Value.Replace(",", ""));
            }
            return 0;
        }

        /// <summary>
        /// Gets one random character
        /// </summary>
        /// <param name="vowel">True if the character is to be vowel, false if consonant.</param>
        /// <param name="rnd"></param>
        /// <returns></returns>
        private char GetChar(bool vowel)
        {
            if (vowel)
            {
                return vowels[random.Next(vowels.Length)];
            }
            return consonants[random.Next(consonants.Length)];
        }
    }
}