﻿#region Using directives

using System;
using System.Drawing;
using System.Collections;
using System.Collections.Specialized;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Forms;
using System.Threading;
using System.Net.Sockets;
using System.Net;
using System.Net.NetworkInformation;
using System.Text.RegularExpressions;

using System.IO;
using System.Text;
using System.Xml;
using System.Configuration;

using Ini;


using System.Diagnostics;
using System.Globalization;

using Heijden.DNS;
#endregion



namespace WhoisLib
{
    public class WhoisSearch : Control
    {

        // Fired when each domain query completes
        public event EventHandler<CellUpdateEventArgs> QueryComplete;

        // Fired when all domain queries complete
        public event EventHandler<LookupCompleteEventArgs> LookupComplete;


        // Limits the number of threads happening simultaniously
        const int THREAD_COUNT = 200;
        private Semaphore queryBlock = new Semaphore(0, THREAD_COUNT);

        private static String appPath = string.Empty;
        private static String strAccessConn = string.Empty;
        private static String strDB = string.Empty;


   		public Resolver resolver;



        public WhoisSearch()
        {
            resolver = new Resolver();
            resolver.OnVerbose += new Resolver.VerboseEventHandler(resolver_OnVerbose);

            //appPath = System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location);
            //strDB = System.IO.Path.Combine(appPath, "multitask.mdb");

            //StringBuilder strBuild = new StringBuilder();
            //strBuild.AppendFormat("Provider={0};Data Source='{1}';", "Microsoft.Jet.OLEDB.4.0", strDB);
            //strAccessConn = strBuild.ToString();

            // Needed before Control.Invoke will work.
            IntPtr i = this.Handle;

            // All items in the semaphore start out locked.  This releases them all.
            queryBlock.Release(THREAD_COUNT);

        }


        private void resolver_OnVerbose(object sender, Resolver.VerboseEventArgs e)
        {
            Console.WriteLine(e.Message);
        }

        //public void DigIt(string name)
        //{
        //    DigIt(name, QType.NS, QClass.IN);
        //}

        //public void DigIt(string name, QType qtype)
        //{
        //    DigIt(name, qtype, QClass.IN);
        //}

        //private delegate void DigItDelegate(string name, QType qtype, QClass qclass);
        //public void BeginDigIt(string name, QType qtype, QClass qclass)
        //{
        //    DigItDelegate d = new DigItDelegate(DigIt);
        //    d.BeginInvoke(name, qtype, qclass, null, null);
        //}






        /// <summary>
        /// Starts query operations on a background thread.
        /// </summary>
        /// <param name="pr">Contains domains to search.</param>
        public void Start(SearchDomains sd)
        {
            Thread t = new Thread(new ParameterizedThreadStart(queryWorker_DoWork));
            t.Start(new SearchDomains(sd.DomainsList));
        }

        /// <summary>
        /// Loops through the domains and does queries.
        /// </summary>
        /// <param name="o">List of domains to query whois.</param>
        private void queryWorker_DoWork(object o)
        {
            SearchDomains searchDomains = (SearchDomains)o;

            // Put each thread spawned in this list to know when they've all completed.
            LinkedList<Thread> threads = new LinkedList<Thread>();

            // Loop through each domain search on a background thread.
            foreach (int key in searchDomains.DomainsList.Keys)
            {
                string zdomain = string.Empty;
                string zserver = string.Empty;
                searchDomains.DomainsList.TryGetValue(key, out zdomain);

                Thread t = new Thread(new ParameterizedThreadStart(DoWhoisQuery));
                object[] objData = new object[] { key, zdomain, zserver };
                t.Start(objData);
                threads.AddLast(t);
                System.Threading.Thread.Sleep(200);
            }

            // Wait until all the queries are done.
            foreach (Thread ln in threads)
            {
                ln.Join();
            }


            // Raise an event saying queries are done on main UI thread.
            string aa = "Complete.";
            try
            {
                ThreadSync.InvokeDelegate(LookupComplete, this, new LookupCompleteEventArgs(aa));
            }
            catch (SystemException e)
            {
                throw e;
            }
            catch (Exception e)
            {

            }  

        }


        /// <summary>
        /// Perform actual queries, and release semaphore so another thread can go.
        /// </summary>
        /// <param name="o">IP address to ping</param>
        private void DoWhoisQuery(object o)
        {
            //public void DigIt(string name, QType qtype, QClass qclass)

            object[] state = (object[])o;
            int    _Row = (int)state[0];
            string _Domain  = (string)state[1];
            string _Server = (string)state[2]; 
            string _Status  = "registered";
            string _DStatus = "unknown";
            string _Info    = string.Empty;

            int iAnswerCount = -1;

            StringBuilder sb = new StringBuilder();

            Thread.CurrentThread.CurrentCulture = new CultureInfo("en-US", false);
            Thread.CurrentThread.CurrentUICulture = new CultureInfo("en-US", false);

            Stopwatch sw = new Stopwatch();

            sw.Start();
            //Response response = resolver.Query(_Domain, QType.NS, QClass.IN);
            Response response = resolver.Query(_Domain, QType.ANY, QClass.IN);
            sw.Stop();

            bool bError = false;
            if (response.Error != "")
            {
                sb.Append(response.Error.ToString());
                iAnswerCount = -1;
                _Status = "unknown";
                _DStatus = "-1";
                _Info = "none";
                bError = true;
            }

            //if (bError)
            //{
            //    sw.Start();
            //    response = resolver.Query(_Domain, QType.CNAME, QClass.ANY);
            //    sw.Stop();
            //    if (response.Error != "")
            //    {
            //        sb.Append(response.Error.ToString());
            //        iAnswerCount = -1;
            //        _Status = "unknown";
            //        _DStatus = "-1";
            //        _Info = "none";
            //        bError = true;
            //    }
            //    else
            //        bError = false;
            //}

            if(!bError)
            {
                iAnswerCount = response.header.ANCOUNT;
                if (iAnswerCount < 1)
                    _Status = "available?";

                if (iAnswerCount > 1)
                    iAnswerCount = 0;

                sb.Append("Got answer:");
                sb.Append("<br />");
                sb.Append(string.Format("->>HEADER<<- opcode: {0}, status: {1}, id: {2}", response.header.OPCODE, response.header.RCODE, response.header.ID));
                sb.Append("<br />");
                sb.Append(string.Format("flags: {0}{1}{2}{3}; QUERY: {4}<br />, ANSWER: {5}<br />, AUTHORITY: {6}<br />, ADDITIONAL: {7}<br />",
                    response.header.QR ? " qr" : "",
                    response.header.AA ? " aa" : "",
                    response.header.RD ? " rd" : "",
                    response.header.RA ? " ra" : "",
                    response.header.QDCOUNT,
                    response.header.ANCOUNT,
                    response.header.NSCOUNT,
                    response.header.ARCOUNT));
                sb.Append("<br />");
                if (response.header.QDCOUNT > 0)
                {
                    sb.Append("QUESTION SECTION:");
                    foreach (Question question in response.Questions)
                        sb.Append(string.Format("{0}", question));
                    sb.Append("<br />");
                }
                if (response.header.ANCOUNT > 0)
                {
                    sb.Append("ANSWER SECTION:");
                    foreach (AnswerRR answerRR in response.Answers)
                        sb.Append(answerRR);
                    sb.Append("<br />");
                }
                if (response.header.NSCOUNT > 0)
                {
                    sb.Append("AUTHORITY SECTION:");
                    foreach (AuthorityRR authorityRR in response.Authorities)
                        Console.WriteLine(authorityRR);
                    sb.Append("<br />");
                }
                if (response.header.ARCOUNT > 0)
                {
                    sb.Append("ADDITIONAL SECTION:");
                    foreach (AdditionalRR additionalRR in response.Additionals)
                        sb.Append(additionalRR);
                    sb.Append("<br />");
                }
                sb.Append(string.Format("Query time: {0} msec", sw.ElapsedMilliseconds));
                sb.Append("<br />");
                sb.Append(string.Format("SERVER: {0}#{1}({2})", response.Server.Address, response.Server.Port, response.Server.Address));
                sb.Append("<br />");
                sb.Append(string.Format("WHEN: " + response.TimeStamp.ToString("ddd MMM dd HH:mm:ss yyyy", new System.Globalization.CultureInfo("en-US"))));
                sb.Append("<br />");
                sb.Append(string.Format("MSG SIZE rcvd: " + response.MessageSize));
                sb.Append("<br />");
            }


            //Can't block here--will throw thread count off!!!
            //queryBlock.Release();

            try
            {
                ThreadSync.InvokeDelegate( QueryComplete, 
                                           this, 
                                           new CellUpdateEventArgs( _Row,
                                                                    _Domain,
                                                                    _Server,
                                                                    _Status,
                                                                    _DStatus,
                                                                    sb.ToString()));
            }
            catch (SystemException e)
            {
                throw e;
            }
            catch (Exception e)
            {

            }  
        }

        //public static bool ConnectionAvailable(string strServer)
        //{
        //    try
        //    {
        //        HttpWebRequest reqFP = (HttpWebRequest)HttpWebRequest.Create(strServer);
        //        HttpWebResponse rspFP = (HttpWebResponse)reqFP.GetResponse();
        //        if (HttpStatusCode.OK == rspFP.StatusCode)
        //        {
        //            // HTTP = 200 - Internet connection available, server online
        //            rspFP.Close();
        //            return true;
        //        }
        //        else
        //        {
        //            // Other status - Server or connection not available
        //            rspFP.Close();
        //            return false;
        //        }
        //    }
        //    catch (WebException)
        //    {
        //        // Exception - connection not available
        //        return false;
        //    }
        //}

    }




    public static class ThreadSync
    {
        public static void InvokeDelegate(Delegate del, params object[] args)
        {
            if (del != null)
            {
                ISynchronizeInvoke synchronizer = del.Target as ISynchronizeInvoke;
                if (synchronizer != null && synchronizer.InvokeRequired)
                {
                    synchronizer.Invoke(del, args);
                    return;
                }
                del.DynamicInvoke(args);
            }
        }
    }//END CLASS WHOISSEARCH


    public class SearchDomains
    {
        private Dictionary<int, string> _domainsList = new Dictionary<int, string>();
        public Dictionary<int, string> DomainsList
        {
            get
            {
                return _domainsList;
            }

            set
            {
                _domainsList = value;
            }
        }

        public SearchDomains(Dictionary<int, string> domainsList)
        {
            _domainsList = new Dictionary<int, string>();
            _domainsList = domainsList;
        }

    }//END CLASS SearchDomains

    
    public class LookupCompleteEventArgs : EventArgs
    {
        public LookupCompleteEventArgs(string FinalResult)
        {
            this._finalResult = FinalResult;
        }

        private string _finalResult;
        public string FinalResult
        {
            get
            {
                return _finalResult;
            }

            set
            {
                _finalResult = value;
            }
        }

    }// END CLASS LookupCompleteEventArgs

    public class CellUpdateEventArgs : EventArgs
    {
        public CellUpdateEventArgs(int Row,
                               string Domain,
                               string Server,
                               string Status,
                               string DStatus,
                               string Info)
        {
            this._row = Row;
            this._whoisDomain = Domain;
            this._whoisServer = Server;
            this._whoisStatus = Status;
            this._whoisDomainStatus = DStatus;
            this._whoisInfo = Info;
        }

        private int _row;
        public int Row
        {
            get
            {
                return _row;
            }

            set
            {
                _row = value;
            }
        }

        private string _whoisServer;
        public string WhoisDomain
        {
            get
            {
                return _whoisDomain;
            }

            set
            {
                _whoisDomain = value;
            }
        }

        private string _whoisDomain;
        public string WhoisServer
        {
            get
            {
                return _whoisServer;
            }

            set
            {
                _whoisServer = value;
            }
        }

        private string _whoisStatus;
        public string WhoisStatus
        {
            get
            {
                return _whoisStatus;
            }

            set
            {
                _whoisStatus = value;
            }
        }

        private string _whoisDomainStatus;
        public string WhoisDomainStatus
        {
            get
            {
                return _whoisDomainStatus;
            }

            set
            {
                _whoisDomainStatus = value;
            }
        }

        private string _whoisInfo;
        public string WhoisInfo
        {
            get
            {
                return _whoisInfo;
            }

            set
            {
                _whoisInfo = value;
            }
        }

    }//END CLASS CellUpdateEventArgs


}
